<?php

/**
 * @file
 * Contains \Drupal\openai_search\Plugin\Block\OpenAiSearchBlock.
 */

namespace Drupal\openai_search\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\block\Annotation\Block;
use Drupal\Core\Annotation\Translation;

/**
 * Provides a simple block.
 *
 * @Block(
 *   id = "openai_search_block",
 *   admin_label = @Translation("OpenAI Search Module Block"),
 *   module = "openai_search"
 * )
 */
class OpenAiSearchBlock extends BlockBase {

  /**
   * Implements \Drupal\block\BlockBase::blockBuild().
   */
  public function build() {
    $config = $this->getConfiguration();

    // Here, we assume you're storing the URL in a 'restful_url' config setting.
    // This could be fetched from your module's configuration like so:
    $restful_url = \Drupal::config('openai_search.settings')->get('restful_url');
    $form = \Drupal::formBuilder()->getForm('Drupal\openai_search\Form\AiSearchForm');
    $form['#attached']['library'][] = 'openai_search/openai_search';
    // Attach drupalSettings
    $node = \Drupal::routeMatch()->getParameter('node');
    $node_id = $node ? $node->id() : NULL;
    $username = NULL;
    $author = $node->getOwner();
    $username = $author ? $author->getAccountName() : NULL;
    $ai_url = $restful_url . '/' . $username . '_' . 'node' . '_' . $node_id;
    $form['#attached']['drupalSettings']['openai_search'] = [
      'restful_url' => $restful_url,
      'ai_url' => $ai_url,
      'node_type' => $node ? $node->bundle() : NULL,
      'session_id' => session_id(),
      'use_javascript' => 1,
      'current_node_id' => $node_id,
    ];

    return $form;
  }
}
